# quoteme 1.0 by k-rist (k-rist@rsn.hk-r.se)
#
# This script basically stores quotes and sends a random quote to the channel
# Inspired by the quotes 1.0 by blue (blue@tical.net)
#
# Only for single channel bots
# Only for 1.3.x bots
#
# Commands:
# quote [line]     - sends a random quote to the channel, if line is
#                   - specified, that line in the quotemefile will be quoted
#                   - first quote is line 1
# addquote <quote> - adds <quote> to the database
# delquote <quote> - attempts to delete <quote>
# lsquotes [start] - lists quotes. If start is specified, start listing at
#                   - that quote, otherwise, start at first quote.
#
# The respones from the commands "addquote", "delquote" and "listquotes" will
# be sent to the user by a notice, if used directly in the channel
#
# The commands "quote", "addquote" and "lsquotes" require the party-line flag (+p)
# to be set. The command "delquote" requires the owner flag (+o) flag set.
# EXCEPTION: The command "quote" in public can be issued by anyone
#
# Using a command in public or msg mode, the command must be preceded by a !
# For example: !quote
# Using a command in dcc mode, the command must be preceded by a .
# For example: .quote
#
# Wishlist:
# Multichannelsupport
# Better errormessages
# User can change access for commands
# Instead of specify how many, specify which quotes that are listed
# Fix "Tcl error dcc_quote: integer value too large to represent" error 
#
###############################################################################

# Where to store the quotes?
set quotemefile ".quoteme"

# Which channel to send quotes to?
set quotemechannel "#ud"

# Allow the !quote command via public? (0/1)
set quotepub 1
# Allow !quote commands via msg? (0/1)
set quotemsg 1
# Allow .quote commands via dcc? (0/1)
set quotedcc 1

# Allow the !addquote command via public? (0/1)
set addquotepub 1
# Allow !addquote commands via msg? (0/1)
set addquotemsg 1
# Allow .addquote commands via dcc? (0/1)
set addquotedcc 1

# Allow the !delquote command via public? (0/1)
set delquotepub 0
# Allow !delquote commands via msg? (0/1)
set delquotemsg 0
# Allow .delquote commands via dcc? (0/1)
set delquotedcc 1

# Allow the !lsquotes command via public? (0/1)
set lsquotespub 1
# Allow !lsquotes commands via msg? (0/1)
set lsquotesmsg 1
# Allow .lsquotes commands via dcc? (0/1)
set lsquotesdcc 1

# Amount of quotes to list when using !lsquotes at public (0/x)
# When a listing occurs in public or msg, all other messages
# sent to public and msg is on hold.
# Setting this to 0 turns it off
set publsamount 5

# Amount of quotes to list when using !lsquotes at msg (0/x)
# When a listing occurs in public or msg, all other messages
# sent to public and msg is on hold.
# Setting this to 0 turns it off
set msglsamount 5

# Amount of quotes to list when using .lsquotes at dcc (0/x)
# This listing does not put other messages on hold
# Setting this to 0 turns it off
set dcclsamount 0

##################################################
# You should NOT change anything below this line #
##################################################

set quotemeversion "quoteme 1.0"

if {[file exists $quotemefile]} {
    if {![file isfile $quotemefile]} {
	putlog "ERROR: $quotemefile is not a file"
        putlog "$quotemeversion not loaded"
        return 0
    }
    if {![file readable $quotemefile]} {
	putlog "ERROR: File $quotemefile is not readable"
	putlog "$quotemeversion not loaded"
	return 0
    }
}

if {$quotepub} { bind pub - !quote pub_quote }
if {$addquotepub} { bind pub p !addquote pub_addquote }
if {$delquotepub} { bind pub o !delquote pub_delquote }
if {$lsquotespub} { bind pub p !lsquotes pub_lsquotes }

if {$quotemsg} { bind msg p !quote msg_quote }
if {$addquotemsg} { bind msg p !addquote msg_addquote }
if {$delquotemsg} { bind msg o !delquote msg_delquote }
if {$lsquotesmsg} { bind msg p !lsquotes msg_lsquotes }

if {$quotedcc} { bind dcc p quote dcc_quote }
if {$addquotedcc} { bind dcc p addquote dcc_addquote }
if {$delquotedcc} { bind dcc o delquote dcc_delquote }
if {$lsquotesdcc} { bind dcc p lsquotes dcc_lsquotes }

proc pub_quote {nick uhost handle channel arg} {
    quote "pub" $nick $arg
    return 1
}

proc pub_addquote {nick uhost handle channel arg}  {
    addquote "pub" $nick $arg
    return 1
}

proc pub_delquote {nick uhost handle channel arg} {
    delquote "pub" $nick $arg 
    return 1
}

proc pub_lsquotes {nick uhost handle channel arg} {
    lsquotes "pub" $nick $arg
    return 1
}

proc msg_quote {nick uhost handle arg} {
    quote "msg" $nick $arg
    return 1
}

proc msg_addquote {nick uhost handle arg} {
    addquote "msg" $nick $arg
    return 1
}

proc msg_delquote {nick uhost handle arg} {
    delquote "msg" $nick $arg
    return 1
}

proc msg_lsquotes {nick uhost handle arg} {
    lsquotes "msg" $nick $arg
    return 1
}

proc dcc_quote {handle idx arg} {
    quote "dcc" $idx $arg
    return 1
}

proc dcc_addquote {handle idx arg}  {
    addquote "dcc" $idx $arg
    return 1
}

proc dcc_delquote {handle idx arg} {
    delquote "dcc" $idx $arg
    return 1
}

proc dcc_lsquotes {handle idx arg} {
    lsquotes "dcc" $idx $arg
    return 1
}

# Sends (if possible) a quote to the channel
# return 1 if ok, 0 otherwise
proc quote {type who arg} {
    global quotemechannel
    global quotemefile
    if {[checkfile $quotemefile "r"]} {
	set quotes [getquotes]
	if {[expr [llength $quotes] > 0]} {
	    ##### Either get a random quote, or a specified quote 
	    if {$arg != ""} {
		set arg [checkarg $arg]
		if {$arg == ""} {
		    sendanswer $type $who "Usage: !quote \[number\]"
		    return 1
		}
		set row [expr $arg - 1]
	    } else {
		set row [rand [llength $quotes]]
	    }

	    ##### Say the quote
	    if {[expr $row >= 0] && [expr $row < [llength $quotes]]} {
		set quote [lindex $quotes $row]
		if {$quote != ""} {
		    set row [expr $row + 1]
		    putserv "PRIVMSG $quotemechannel :\002($row) $quote\002" 
		    return 1
		}
	    }
	}
    }
    sendanswer $type $who "quote failed"
    return 0
}

proc addquote {type who arg} {
    global quotemefile
    if {[file exists $quotemefile]} {
	if {![file writable $quotemefile] || [file isdir $quotemefile]} {
	    sendanswer $type $who "addquote failed"
	    return 0
	}
    }

    if {$arg != ""} {
	set quotes [getquotes]
	if {![expr [lsearch -exact $quotes $arg] >= 0]} {
	    set fileio [open $quotemefile "a"]
	    puts $fileio $arg
	    flush $fileio
	    close $fileio
	    sendanswer $type $who "Quote added."
	    return 1
	} else {
	    sendanswer $type $who "Quote already exists."
	    return 0
	}
    }
    sendanswer $type $who "Usage: !addquote <quote>"
    return 0
}

proc delquote {type who arg} {
    global quotemefile
    if {[checkfile $quotemefile "w"]} {
	if {$arg != ""} {
	    set arg [checkarg $arg]
	}
	if {$arg == ""} {
	    sendanswer $type $who "Usage: !delqoute <number>"
	    return 0
	}

	set row [expr $arg - 1]
        set oldquotes [getquotes]
	if {[expr $row >= 0] && [expr $row < [llength $oldquotes]]} {
	    set newquotes [lreplace $oldquotes $row $row]
	    set fileio [open $quotemefile "w"]
	    foreach quote $newquotes {
		puts $fileio $quote
	    }
	    flush $fileio
	    close $fileio
	    sendanswer $type $who "Quote deleted"
	    return 1
	}
    }
    sendanswer $type $who "delquote failed"
    return 0
}

proc lsquotes {type who arg} {
    global quotemefile
    global publsamount
    global msglsamount
    global dcclsamount
    if {[checkfile $quotemefile "r"]} {
	set quotes [getquotes]
	set newquotes ""
	foreach quote $quotes {
	    set index [lsearch -exact $quotes $quote]
	    set row [expr $index + 1]
	    set newquote [concat $row $quote]
	    set newquotes [linsert $newquotes end $newquote]
	}

	if {$arg != ""} {
	    set arg [checkarg $arg]
	    if {$arg == ""} {
		sendanswer $type $who "Usage: !lsquote \[number\]"
                return 1
            }
	    if {[expr $arg > [llength $newquotes]]} {
		set newquotes ""
	    } else {
		#   Since quotes start at 1, and we also want to list the specified
		#   quote, we hade to start removing from -2 and down.
		set arg [expr $arg - 2]
		while {[expr $arg >= 0]} {
		    set newquotes [lreplace $newquotes $arg $arg]
		    set arg [expr $arg - 1]
		}
	    }
	}

	if {$type == "pub"} {
	    set lsamount $publsamount
	} elseif {$type == "msg"} {
	    set lsamount $msglsamount
	} elseif {$type == "dcc"} {
	    set lsamount $dcclsamount
	} else {
	    return 0
	}
	set quotesamount [llength $newquotes]
	if {[expr $quotesamount > $lsamount] && [expr $lsamount > 0]} {
	    set quotesamount $lsamount
	}
	sendanswer $type $who "Quote list ([getamount] total, listing $quotesamount):"
	#Only list $lsamount amount of quotes
	set index $lsamount
	if {[expr $index <= 0] || [expr $index > $quotesamount]} {
	    set index $quotesamount
	}
	foreach quote $newquotes {
	    if {[expr $index > 0]} {
		sendanswer $type $who "$quote"
		set index [expr $index - 1]
	    } else {
		break
	    }
	}
	sendanswer $type $who "End of quotes list"
	return 1
    }
    sendanswer $type $who "lsquotes failed"
    return 0
}

# Checks if an arg is numeric (truncates the arg to first character)
# arg  - the arg to check
# return the new arg if arg is ok, "" if it failed
proc checkarg {arg} {
    set arg [lindex $arg 0]
    if {[regexp "^\[0-9\]+$" $arg]} {
	return $arg
    }
    return ""
}

# Checks if the file exists and is a file (and not a directory)
# file - the file to check
# rw   - set "r" if file should be checked for readability
#      - set "w" if file should be checked for writability
# return 1 if file is ok, 0 if it failed
proc checkfile {file rw} {
    if {[file exists $file] && [file isfile $file]} {
	if {$rw == "r"} {
	    if {[file readable $file]} {
		return 1
	    } else {
		return 0
	    }
	} elseif {$rw == "w"} {
	    if {[file writable $file]} {
		return 1
	    } else {
		return 0
	    }
	} else {
	    return 1
	}
    }
    return 0
}

# Returns the amount of quotes if possible, 0 otherwise
proc getamount {} {
    global quotemefile
    if {[checkfile $quotemefile "r"]} {
	set quotes [getquotes]
	return [llength $quotes]
    }
    return 0
}

# Returns a list with the quotes
proc getquotes {} {
    global quotemefile
    set quotes ""
    set fileio [open $quotemefile "r"]
    while {![eof $fileio]} {
	set quote [gets $fileio]
	if {$quote != ""} {
            set quotes [linsert $quotes end $quote]
	}
    }
    close $fileio
    return $quotes
}

# Sends a message to the specified user
# return 1 if ok, 0 otherwise
proc sendanswer {type who message} {
    if {$type == "pub"} {
	set message [concat ":" $message]
	puthelp "NOTICE $who $message"
    } elseif {$type == "msg"} {
	set message [concat ":" $message]
	puthelp "PRIVMSG $who $message"
    } elseif {$type == "dcc"} {
	putdcc $who $message
    } else {
	return 0
    }
    return 1
}

putlog "$quotemeversion loaded."
